@@ -138,18 +138,98 @@
   ;; display-graphic-p
   (and window-system (image-type-available-p format)))
 
+(defun mew-jpeg-width ()
+  (let (c size width)
+    (save-excursion
+      (catch 'loop
+	(while t
+	  (setq c (char-after))
+	  (forward-char)
+	  (unless (eq c #xff)
+	    (throw 'loop nil))
+	  (setq c (char-after))
+	  (forward-char)
+	  (cond
+	   ((eq c #xd8)
+	    )
+	   ((eq c #xe0)
+	    (forward-char 16))
+	   ((or (eq c #xdb) (eq c #xc4)
+		(and (>= c #xe0) (<= c #xed)))
+	    (setq size (char-after))
+	    (forward-char)
+	    (setq size (+ (* size 256) (char-after)))
+	    (forward-char)
+	    (forward-char (- size 2)))
+	   ((eq c #xc0)
+	    (forward-char 5)
+	    (setq width (char-after))
+	    (forward-char)
+	    (setq width (+ (* width 256) (char-after)))
+	    (throw 'loop nil))
+	   (t
+	    (throw 'loop nil))))))
+    width))
+
+(defun mew-png-width ()
+  (let (width)
+    (save-excursion
+      (forward-char 18)
+      ;; length is four bytes
+      ;; but we takes lower two bytes
+      (setq width (char-after))
+      (forward-char)
+      (setq width (+ (* width 256) (char-after)))
+      width)))
+
+(defvar mew-image-width-margin 45)
+
+(defvar mew-image-alist
+  '((jpeg "jpegtopnm" mew-jpeg-width)
+    (png  "pngtopnm"  mew-png-width)))
+
+(defun mew-image-format-ent (format)
+  (assoc format mew-image-alist))
+
+(defun mew-image-get-prog (ent)
+  (nth 1 ent))
+
+(defun mew-image-get-func (ent)
+  (nth 2 ent))
+
 (defun mew-mime-image (cache begin end format)
   (message "Loading image...")
   (set-buffer (mew-buffer-message))
-  (mew-elet
-   (condition-case nil
-       (insert-image
-	(create-image (save-excursion
-			(set-buffer cache)
-			(string-as-unibyte
-			 (mew-buffer-substring begin end)))
-		      format t))
-     (error ())))
+  (let* ((width (- (frame-pixel-width (selected-frame)) mew-image-width-margin))
+	 (ent (mew-image-format-ent format))
+	 (prog (mew-image-get-prog ent))
+	 (func (mew-image-get-func ent))
+	 image-width image)
+    (when (and func (fboundp func))
+      (save-excursion
+	(set-buffer cache)
+	(goto-char begin)
+	(setq image-width (funcall func))))
+    (if (and image-width (< width image-width) (mew-which-exec prog))
+	(with-temp-buffer
+	  (message "Resizing image...")
+	  (insert-buffer-substring cache begin end)
+	  (mew-set-buffer-multibyte nil)
+	  (call-process-region (point-min) (point-max) prog
+			       t '(t nil) nil)
+	  (call-process-region (point-min) (point-max) "pnmscale"
+			       t '(t nil) nil
+			       "-xsize" (format "%d" width))
+	  (setq format 'pbm)
+	  (setq image (mew-buffer-substring (point-min) (point-max)))
+	  (message "Resizing image...done"))
+      (save-excursion
+	(set-buffer cache)
+	(setq image (mew-buffer-substring begin end))))
+    (mew-elet
+     (condition-case nil
+	 (insert-image (create-image (string-as-unibyte image) format t))
+       (error ()))))
   (goto-char (point-min))
   (message "Loading image...done"))
 
@@ -212,11 +292,25 @@
 				(find-image '((:type xpm :file "mew-lock.xpm" :ascent center))))))
     " [Sec]"))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Click hack
+;;;
+
+(defun mew-browse-url-at-mouse (event)
+  (interactive "e")
+  (let ((buf (current-buffer)))
+    (browse-url-at-mouse event)
+    (pop-to-buffer buf)))
+
+(define-key mew-message-mode-map [mouse-2] 'mew-browse-url-at-mouse)
+(define-key mew-message-mode-map [M-down-mouse-1] 'mew-browse-url-at-mouse) ;; for Mac
+
 (provide 'mew-gemacs)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2005 Mew developing team.
+;; Copyright (C) 1997-2006 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
