@@ -36,8 +36,9 @@
     "delete"
     "case" "mailbox" "msgid" "max" "tag"
     "wrk" "jobs" "namespace" "authl"
-    "spam" "spam-field" "spam-word" "new-mailbox"
-    "my-prefix" "specials" "bytes"))
+    "spam" "spam-field" "spam-word" "spam-pattern" 
+    "new-mailbox" "my-prefix" "specials" "bytes"
+    "thread-info" "disp-info" "status-buf"))
 
 (mew-info-defun "mew-imap-" mew-imap-info-list)
 
@@ -92,7 +93,7 @@
   (or (mew-imap-get-ssh-process pnm) (mew-imap-get-ssl-process pnm)))
  
 (defun mew-imap-command-capability (pro pnm)
-  (mew-net-status (mew-imap-get-bnm pnm)
+  (mew-net-status (mew-imap-get-status-buf pnm)
 		  "Auth'ing"
 		  nil
 		  (mew-imap-secure-p pnm))
@@ -150,7 +151,7 @@
     (when (and (eq directive 'jobs) jobs)
       (let* ((case (mew-imap-get-case pnm))
 	     (bnm (mew-imap-get-bnm pnm))
-	     (file (mew-expand-folder bnm (car jobs)))
+	     (file (mew-expand-msg bnm (car jobs)))
 	     (work (concat file mew-queue-work-suffix))
 	     (data (concat file mew-imapq-info-suffix))
 	     (data-work (concat data mew-queue-work-suffix))
@@ -159,6 +160,7 @@
 	(setq info (mew-lisp-load data))
 	(rename-file data data-work 'override)
 	(setq mailbox (car info))
+	(setq mailbox (mew-imap-utf-7-encode-string mailbox))
 	(setq mailbox (mew-imap-expand-mailbox case mailbox))
 	(mew-imap-set-mailbox pnm mailbox)
 	(setq info (cdr info))
@@ -204,16 +206,17 @@
     (if (and mew-imap-skip-uidl
 	     (or (eq directive 'exec) (eq directive 'jobs) (eq directive 'get)))
 	(mew-imap-command-dispatch pro pnm directive refs rmvs nil)
-      (mew-net-status bnm "Checking" nil (mew-imap-secure-p pnm))
+      (mew-net-status (mew-imap-get-status-buf pnm)
+		      "Checking" nil (mew-imap-secure-p pnm))
       (cond
        ((eq directive 'biff)
-	(setq max (mew-lisp-load (mew-expand-folder bnm mew-imap-msgid-file))))
+	(setq max (mew-lisp-load (mew-expand-file bnm mew-imap-msgid-file))))
        ((eq directive 'scan)
 	(if (eq range nil) ;; update
 	    (setq max (mew-lisp-load
-		       (mew-expand-folder bnm mew-imap-msgid-file))))))
+		       (mew-expand-file bnm mew-imap-msgid-file))))))
       (if max
-	  (setq max (string-to-int max))
+	  (setq max (string-to-number max))
 	(setq max 0))
       (mew-imap-set-max pnm max)
       (mew-imap-process-send-string
@@ -242,11 +245,11 @@
 	 ((member uid rmvs)
 	  (setq dels (cons uid dels)))))
        ((eq directive 'biff)
-	(if (and uid (> (string-to-int uid) max))
+	(if (and uid (> (string-to-number uid) max))
 	    (setq rtrs (cons (list uid siz) rtrs))))
        ((eq directive 'scan)
 	(if (and uid (or range ;; all, last:n
-			 (> (string-to-int uid) max))) ;; update
+			 (> (string-to-number uid) max))) ;; update
 	    (setq rtrs (cons (list uid siz) rtrs))))
        ((eq directive 'sync)
 	(if uid (setq hlds (cons uid hlds))))
@@ -328,7 +331,10 @@
     (mew-imap-set-rttl pnm (length rtrs))
     (mew-imap-set-rtrs pnm rtrs)
     (mew-imap-set-dttl pnm (length dels))
-    (setq dels (mew-net-msg-group dels))))
+    ;; We can't use mew-net-msg-group since a member will be
+    ;; deleted from dels.
+    ;; (setq dels (mew-net-msg-group dels))
+    ))
   ;;
   (mew-imap-set-dels pnm dels)
   (mew-imap-set-dgttl pnm (length dels))
@@ -377,7 +383,7 @@
 
 (defun mew-imap-command-copy (pro pnm)
   (mew-net-status1
-   (mew-imap-get-bnm pnm) "Refiling"
+   (mew-imap-get-status-buf pnm) "Refiling"
    (mew-imap-get-rgttl pnm) (mew-imap-get-rgcnt pnm) (mew-imap-secure-p pnm))
   (let* ((case (mew-imap-get-case pnm))
 	 (spam (mew-imap-get-spam pnm))
@@ -395,7 +401,8 @@
 	  (mew-imap-command-logout pro pnm)))
       (mew-imap-set-rgcnt pnm (1+ (mew-imap-get-rgcnt pnm)))
       (mew-imap-set-rtrs pnm (cdr rtrs))
-      (setq dst (mew-imap-expand-mailbox case dst))
+      (setq dst (mew-imap-expand-mailbox
+		 case (mew-imap-utf-7-encode-string dst)))
       (mew-imap-set-rfl pnm rtr)
       (if spam
 	  (mew-imap-process-send-string pro pnm "COPY %s \"%s\"" uid dst)
@@ -408,7 +415,8 @@
 	 (dst (nth 0 rtr))) ;; 'exec or 'jobs
     (mew-imap-set-rgcnt pnm (1- (mew-imap-get-rgcnt pnm)))
     (mew-imap-set-rtrs pnm (cons rtr rtrs))
-    (setq dst (mew-imap-expand-mailbox case dst))
+    (setq dst (mew-imap-expand-mailbox
+	       case (mew-imap-utf-7-encode-string dst)))
     (mew-imap-process-send-string pro pnm "CREATE \"%s\"" dst)))
 
 (defun mew-imap-command-wmbx (pro pnm)
@@ -443,7 +451,7 @@
 
 (defun mew-imap-command-dels (pro pnm)
   (mew-net-status1
-   (mew-imap-get-bnm pnm) "Deleting"
+   (mew-imap-get-status-buf pnm) "Deleting"
    (mew-imap-get-dgttl pnm) (mew-imap-get-dgcnt pnm) (mew-imap-secure-p pnm))
   (let ((dels (mew-imap-get-dels pnm))
 	(spam (mew-imap-get-spam pnm))
@@ -466,11 +474,14 @@
 
 (defun mew-imap-command-search (pro pnm)
   (mew-net-status
-   (mew-imap-get-bnm pnm) "Filtering" nil (mew-imap-secure-p pnm))
-  (mew-imap-process-send-string
-   pro pnm "SEARCH HEADER %s %s"
-   (mew-imap-get-spam-field pnm)
-   (mew-imap-get-spam-word pnm)))
+   (mew-imap-get-status-buf pnm) "Filtering" nil (mew-imap-secure-p pnm))
+  (let ((pattern (mew-imap-get-spam-pattern pnm)))
+    (if pattern
+	(mew-imap-process-send-string pro pnm "SEARCH %s" pattern)
+      (mew-imap-process-send-string
+       pro pnm "SEARCH HEADER %s %s"
+       (mew-imap-get-spam-field pnm)
+       (mew-imap-get-spam-word pnm)))))
 
 (defun mew-imap-command-post-search (pro pnm)
   (let* ((case (mew-imap-get-case pnm))
@@ -524,7 +535,7 @@
       (mew-imap-command-fetch pro pnm)))))
 
 (defun mew-imap-command-fetch (pro pnm)
-  (mew-net-status2 (mew-imap-get-bnm pnm)
+  (mew-net-status2 (mew-imap-get-status-buf pnm)
 		   (mew-imap-get-rttl pnm)
 		   (mew-imap-get-rcnt pnm)
 		   (nth 1 (car (mew-imap-get-rtrs pnm)))
@@ -549,17 +560,19 @@
 	(mew-imap-command-logout pro pnm)))
      ((or (eq directive 'exec) (eq directive 'jobs) (eq directive 'get))
       (mew-imap-set-truncated pnm nil)
+      ;; RFC822 turns the Seen flag "on"
       (mew-imap-process-send-string pro pnm "UID FETCH %s RFC822" uid))
      ((and (eq directive 'scan) (not get-body))
       (mew-imap-set-truncated pnm t)
-      (mew-imap-process-send-string pro pnm "UID FETCH %s RFC822.HEADER" uid))
-     ((or (= lim 0) (< (string-to-int siz) lim))
+      ;; RFC822 keeps the Seen flag "off"
+      (mew-imap-process-send-string pro pnm "UID FETCH %s BODY.PEEK[HEADER]" uid))
+     ((or (= lim 0) (< (string-to-number siz) lim))
       (mew-imap-set-truncated pnm nil)
       (mew-imap-process-send-string pro pnm "UID FETCH %s RFC822" uid))
      (t
       (mew-imap-set-truncated pnm t)
       (mew-imap-process-send-string
-       pro pnm "UID FETCH %s RFC822.HEADER" uid)))))
+       pro pnm "UID FETCH %s BODY.PEEK[HEADER]" uid)))))
 
 (defun mew-imap-command-post-fetch (pro pnm)
   (let* ((directive (mew-imap-get-directive pnm))
@@ -568,9 +581,11 @@
 	 (rtr (car rtrs))
 	 (uid (nth 0 rtr))
 	 (siz (nth 1 rtr))
+	 (del (nth 2 rtr))
 	 (fld-msg (nth 3 rtr)) ;; 'exec, 'jobs, 'scan, ...
 	 (truncated (mew-imap-get-truncated pnm))
-	 fld msg vec file msg-file lmsg mark)
+	 (case (mew-imap-get-case pnm))
+	 fld msg vec file msg-file lmsg mark folder)
     (cond
      ((null fld-msg)
       (setq fld (mew-imap-get-bnm pnm)))
@@ -579,13 +594,14 @@
      ((listp fld-msg)
       (mew-set '(fld msg) fld-msg)
       (setq lmsg msg)))
+    (setq folder (mew-case:folder-folder fld))
     ;; line delimiters
     (goto-char (point-min))
     (mew-crlf-to-lf)
     (cond
      ((eq directive 'scan)
       (setq msg uid) 
-      (setq file (mew-expand-folder fld msg)))
+      (setq file (mew-expand-new-msg fld msg)))
      (t
       (setq msg-file (mew-net-get-new-message
 		      pnm fld msg 'mew-imap-get-msgdb 'mew-imap-set-msgdb))
@@ -593,10 +609,12 @@
     (goto-char (point-min))
     (cond
      (truncated
-      (mew-header-insert-xmu uid siz t))
+      (mew-header-insert-xmu uid siz t (and (eq directive 'inc)
+					    (or (mew-imap-get-case pnm)
+						mew-case-default))))
      ((eq directive 'scan)
       (mew-header-insert-xmu uid siz nil))
-     ((and (eq directive 'get) (mew-folder-imapp (mew-case:folder-folder fld)))
+     ((and (eq directive 'get) (mew-folder-imapp folder))
       (mew-header-insert-xmu uid siz nil)))
     (catch 'write-error
       (condition-case nil
@@ -607,20 +625,29 @@
 	 (mew-imap-set-status pnm "logout")
 	 (mew-imap-command-logout pro pnm)
 	 (throw 'write-error nil)))
-      (mew-set-file-modes file)
       (mew-imap-set-rcnt pnm (1+ (mew-imap-get-rcnt pnm)))
-      ;;
-      (mew-set-buffer-multibyte t)
-      (setq vec (mew-scan-header))
-      (mew-scan-set-folder vec fld)
-      (mew-scan-set-message vec msg)
-      (if (and (eq directive 'scan)
-	       (string= (mew-case:folder-folder fld) mew-imap-inbox-folder))
-	  (setq mark (mew-scan-inbox-action vec)))
-      (mew-scan-body vec)
-      (mew-set-buffer-multibyte nil)
-      (mew-scan-insert-line fld vec width lmsg mark)
+      (when (file-exists-p file)
+	(mew-set-file-modes file)
+	(unless (string= folder (mew-imap-trash-folder case))
+	  (mew-set-file-type file))
+	;;
+	(mew-set-buffer-multibyte t)
+	(setq vec (mew-scan-header))
+	(mew-scan-set-folder vec fld)
+	(mew-scan-set-message vec msg)
+	(if (and (eq directive 'scan)
+		 (string= folder mew-imap-inbox-folder))
+	    (setq mark (mew-scan-inbox-action vec case)))
+	(mew-scan-body vec)
+	(mew-set-buffer-multibyte nil)
+	(mew-scan-insert-line fld vec width lmsg mark))
       (mew-imap-set-rtrs pnm (cdr rtrs))
+      (cond
+       ((eq directive 'inc)
+	(if truncated
+	    (mew-imap-set-dels pnm (delete uid (mew-imap-get-dels pnm)))))
+       ((eq directive 'get)
+	(if del (mew-imap-set-dels pnm (cons uid (mew-imap-get-dels pnm))))))
       (mew-imap-set-status pnm "fetch")
       (mew-imap-set-bytes pnm nil)
       (mew-imap-command-fetch pro pnm))))
@@ -689,13 +716,15 @@
   (mew-imap-command-list pro pnm))
 
 (defun mew-imap-command-list (pro pnm)
-  (mew-net-status (mew-imap-get-bnm pnm)
+  (mew-net-status (mew-imap-get-status-buf pnm)
 		  "Listing"
 		  nil
 		  (mew-imap-secure-p pnm))
-  (let ((my-prefix (mew-imap-get-my-prefix pnm)))
+  (let* ((my-prefix (mew-imap-get-my-prefix pnm))
+	 (case (mew-imap-get-case pnm))
+	 (prefix-list (mew-imap-prefix-list case)))
     (mew-imap-message pnm "Collecting mailbox list...")
-    (mew-imap-process-send-string pro pnm "LIST \"%s\" *" (or my-prefix ""))))
+    (mew-imap-process-send-string pro pnm "LIST \"%s\" *" (or (and prefix-list my-prefix) ""))))
 
 (defun mew-imap-command-post-list (pro pnm)
   (let* ((case (mew-imap-get-case pnm))
@@ -739,6 +768,7 @@
 	    (setq subfolder (format "\\([^%s\n\r]+\\)$" sep))
 	    (setq friendp (mew-imap-friend-regex sep case)))
 	  (setq mbx (mew-match-string 2))
+	  (setq mbx (mew-imap-utf-7-decode-string mbx))
 	  (cond
 	   ((mew-imap-inbox-str-p mbx)
 	    (throw 'break nil))
@@ -790,7 +820,7 @@
       (setq len1 len2))))
 
 (defun mew-imap-command-delete (pro pnm)
-  (mew-net-status (mew-imap-get-bnm pnm)
+  (mew-net-status (mew-imap-get-status-buf pnm)
 		  "Deleting"
 		  nil
 		  (mew-imap-secure-p pnm))
@@ -799,7 +829,7 @@
     (mew-imap-process-send-string pro pnm "DELETE \"%s\"" mailbox)))
 
 (defun mew-imap-command-rename (pro pnm)
-  (mew-net-status (mew-imap-get-bnm pnm)
+  (mew-net-status (mew-imap-get-status-buf pnm)
 		  "Renaming"
 		  nil
 		  (mew-imap-secure-p pnm))
@@ -1004,7 +1034,8 @@
 ;;;
 
 (defsubst mew-imap-bnm-to-mailbox (bnm)
-  "case:%inbox -> inbox
+  "
+   case:%inbox -> inbox
    case:%foo   -> inbox.foo
    case:%~foo  -> user.foo
    case:%#foo  -> foo"
@@ -1037,11 +1068,12 @@
 	(if my-prefix (concat my-prefix str) str))))))
 
 (defun mew-imap-expand-folder (case str)
-  "foo   -> foo
+  "
+   foo   -> foo
    ~foo  -> user.foo
    #foo  -> #foo
 "
-  (when (stringp str)
+  (when (and (stringp str) (not (string= str "")))
     (if (char-equal (aref str 0) ?~)
 	(let* ((namespace (mew-imap-namespace case))
 	       (user-prefix (mew-imap-namespace-user-prefix namespace)))
@@ -1100,7 +1132,7 @@
 	pro tm)
     (condition-case emsg
 	(progn
-	  (setq tm (mew-timer mew-imap-timeout-time 'mew-imap-timeout))
+	  (setq tm (run-at-time mew-imap-timeout-time nil 'mew-imap-timeout))
 	  (or no-msg (message "Connecting to the IMAP server..."))
 	  (setq pro (open-network-stream pnm nil server sprt))
 	  (process-kill-without-query pro)
@@ -1131,11 +1163,14 @@
 	 (sslp (mew-imap-ssl case))
 	 (sslport (mew-imap-ssl-port case))
 	 ;; dirty but necessary for migration
-	 (mailbox (unless (eq directive 'list) (mew-imap-bnm-to-mailbox bnm)))
+	 (mailbox (unless (eq directive 'list)
+		    (mew-imap-utf-7-encode-string
+		     (mew-imap-bnm-to-mailbox bnm))))
 	 (pnm (mew-imap-info-name case mailbox))
 	 (buf (get-buffer-create (mew-imap-buffer-name pnm)))
 	 (no-msg (eq directive 'biff))
-	 process sshname sshpro sslname sslpro lport info jobs)
+	 process sshname sshpro sslname sslpro lport info jobs tls
+	 thread-info disp-info virtual)
     (if (mew-imap-get-process pnm)
 	(message "Another IMAP process is running. Try later")
       (cond
@@ -1147,12 +1182,13 @@
 	  (when lport
 	    (setq process (mew-imap-open pnm "localhost" lport no-msg)))))
        (sslp
- 	(setq sslpro (mew-open-ssl-stream server sslport))
+	(if (string= port sslport) (setq tls mew-tls-imap))
+ 	(setq sslpro (mew-open-ssl-stream case server sslport tls))
  	(when sslpro
  	  (setq sslname (process-name sslpro))
 	  (setq lport (mew-ssl-pnm-to-lport sslname))
  	  (when lport
- 	    (setq process (mew-imap-open pnm "localhost" lport no-msg)))))
+ 	    (setq process (mew-imap-open pnm mew-ssl-localhost lport no-msg)))))
        (t
 	(setq process (mew-imap-open pnm server port no-msg))))
       (if (null process)
@@ -1162,6 +1198,7 @@
 	(mew-sinfo-set-summary-form (mew-get-summary-form bnm))
 	(mew-sinfo-set-unread-mark nil)
 	(mew-sinfo-set-scan-id nil)
+	(mew-sinfo-set-scan-md5 nil)
 	(mew-info-clean-up pnm)
 	(mew-imap-set-no-msg pnm no-msg) ;; must come here
 	(mew-imap-message pnm "Communicating with the IMAP server...")
@@ -1177,6 +1214,7 @@
 	(mew-imap-set-status pnm "greeting")
 	(mew-imap-set-directive pnm directive)
 	(mew-imap-set-bnm pnm bnm)
+	(mew-imap-set-status-buf pnm bnm)
 	(mew-imap-set-rcnt pnm 1)
 	(mew-imap-set-rgcnt pnm 1)
 	(mew-imap-set-dgcnt pnm 1)
@@ -1196,7 +1234,17 @@
 	  (mew-imap-set-delete pnm (mew-imap-delete case)))
 	 ((eq directive 'get)
 	  (mew-sinfo-set-unread-mark (mew-get-unread-mark bnm))
-	  (mew-imap-set-refs pnm (nth 0 args)))
+	  (mew-imap-set-refs pnm (nth 0 args))
+	  (setq thread-info (nth 1 args))
+	  (mew-imap-set-thread-info pnm thread-info)
+	  (setq disp-info (nth 2 args))
+	  (mew-imap-set-disp-info pnm disp-info)
+	  (setq virtual (mew-net-thread-info-get-virtual thread-info))
+	  (when virtual
+	    (mew-imap-set-status-buf pnm virtual)
+	    (save-excursion
+	      (set-buffer virtual)
+	      (mew-summary-lock process "IMAPing" (or sshpro sslpro)))))
 	 ((eq directive 'scan)
 	  (mew-imap-set-range pnm (nth 0 args))
 	  (mew-imap-set-get-body pnm (nth 1 args))
@@ -1205,12 +1253,16 @@
 		(mew-imap-set-mdb pnm (mew-summary-mark-collect4))
 		(mew-net-folder-clean))
 	    (mew-sinfo-set-unread-mark (mew-get-unread-mark bnm))
-	    (mew-net-invalid-cache-clean))
+	    (mew-imap-set-mdb pnm (mew-summary-mark-collect5))
+	    (mew-net-invalid-cache-invisible))
 	  (when (and (string= mailbox mew-imap-inbox-string)
-		     (mew-imap-spam-field case) (mew-imap-spam-word case))
+		     (or (mew-imap-spam-pattern case)
+			 (and (mew-imap-spam-field case)
+			      (mew-imap-spam-word case))))
 	    (mew-imap-set-spam pnm t)
 	    (mew-imap-set-spam-field pnm (mew-imap-spam-field case))
-	    (mew-imap-set-spam-word pnm (mew-imap-spam-word case))))
+	    (mew-imap-set-spam-word pnm (mew-imap-spam-word case))
+	    (mew-imap-set-spam-pattern pnm (mew-imap-spam-pattern case))))
 	 ((eq directive 'sync)
 	  )
 	 ((eq directive 'exec)
@@ -1252,7 +1304,11 @@
     (save-excursion
       (set-buffer (get-buffer-create mew-buffer-debug))
       (goto-char (point-max))
-      (insert (format "\n<%s>\n%s\n" label string)))))
+      (let ((start (point)))
+	(insert (format "\n<%s>\n%s\n" label string))
+	(goto-char start)
+	(mew-crlf-to-lf))
+      (goto-char (point-max)))))
 
 (defun mew-imap-filter (process string)
   (let* ((pnm (process-name process))
@@ -1269,7 +1325,7 @@
      (insert string)
      (cond
       ((string= status "fetch")
-       (mew-net-status2 (mew-imap-get-bnm pnm)
+       (mew-net-status2 (mew-imap-get-status-buf pnm)
 			(mew-imap-get-rttl pnm)
 			(mew-imap-get-rcnt pnm)
 			(nth 1 (car (mew-imap-get-rtrs pnm)))
@@ -1278,7 +1334,7 @@
        (goto-char (point-min))
        (when (and (null bytes)
 		  (looking-at "^\\*[^{]*{\\([0-9]+\\)}"))
-	 (setq bytes (string-to-int (mew-match-string 1)))
+	 (setq bytes (string-to-number (mew-match-string 1)))
 	 (mew-imap-set-bytes pnm bytes))
        (when bytes
 	 (goto-char (point-min))
@@ -1342,8 +1398,10 @@
 	 (msgid (mew-imap-get-msgid pnm))
 	 (done (mew-imap-get-done pnm))
 	 (error (mew-imap-get-error pnm))
-	 (file (mew-expand-folder bnm mew-imap-msgid-file))
-	 (buf (process-buffer process)))
+	 (file (mew-expand-file bnm mew-imap-msgid-file))
+	 (buf (process-buffer process))
+	 (thread-info (mew-imap-get-thread-info pnm))
+	 (disp-info (mew-imap-get-disp-info pnm)))
     (mew-imap-debug "IMAP SENTINEL" event)
     (set-process-buffer process nil)
     (set-buffer bnm)
@@ -1360,13 +1418,15 @@
 	  (cond
 	   ((eq directive 'exec)
 	    (mew-imap-exec-recover bnm))
+	   ((eq directive 'scan)
+	    (mew-summary-visible-buffer bnm))
 	   ((or (eq directive 'delete) (eq directive 'rename))
 	    (setq mew-summary-buffer-process-error t)))
 	  (when (mew-imap-get-dispatched pnm)
 	    (cond
 	     ((eq directive 'scan)
 	      (when msgid
-		(setq msgid (int-to-string (1- (string-to-int msgid))))
+		(setq msgid (number-to-string (1- (string-to-number msgid))))
 		(mew-lisp-save file msgid nil 'unlimit))
 	      (setq leftp t))
 	     ((eq directive 'inc)
@@ -1378,9 +1438,10 @@
 	    (when leftp
 	      (mew-imap-message
 	       pnm
-	       "%d message retrieved. %d messages are left due to an error."
+	       "%d message retrieved. %d messages are left due to an error"
 	       (- rttl lefts) lefts)
 	      (mew-summary-folder-cache-save))))
+      (if thread-info (mew-summary-retrieve-message-for-thread thread-info))
       (cond
        ((eq directive 'list)
 	(mew-imap-message pnm "Collecting mailbox list...done"))
@@ -1425,11 +1486,11 @@
 	 ((= rttl 1)
 	  (mew-imap-message pnm "1 message retrieved")
 	  (mew-lisp-save file msgid nil 'unlimit)
-	  (mew-summary-folder-cache-save))
+	  (mew-net-invalid-cache-clean))
 	 (t
 	  (mew-imap-message pnm "%s messages retrieved" rttl)
 	  (mew-lisp-save file msgid nil 'unlimit)
-	  (mew-summary-folder-cache-save))))
+	  (mew-net-invalid-cache-clean))))
        ((eq directive 'exec)
 	(if kils (mew-mark-exec-unlink bnm kils))
 	(when movs
@@ -1455,7 +1516,7 @@
 	    (mew-imap-message pnm "1 job processed")
 	  (mew-imap-message pnm "%d jobs processed" jcnt)))))
     ;;
-    (mew-net-status-clear bnm)
+    (mew-net-status-clear (mew-imap-get-status-buf pnm))
     (mew-info-clean-up pnm)
     (set-buffer-modified-p nil)
     (mew-summary-unlock)
@@ -1463,11 +1524,12 @@
 	(process-send-string sshpro "exit\n"))
     (if (and (processp sslpro) (not mew-ssl-keep-connection))
  	 (delete-process sslpro))
+    (mew-net-disp-info-display disp-info)
     (unless (eq directive 'biff)
       (run-hooks 'mew-imap-sentinel-non-biff-hook))
     (run-hooks 'mew-imap-sentinel-hook)
-     (when (and mew-auto-flush-queue flush)
-       (mew-smtp-flush-queue mew-case-output))))
+    (when (and mew-auto-flush-queue flush)
+      (mew-smtp-flush-queue mew-case))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -1475,7 +1537,7 @@
 ;;;
 
 (defun mew-imap-biff ()
-  (let* ((case mew-case-input)
+  (let* ((case mew-case)
 	 (inbox (mew-proto-inbox-folder nil case))
 	 (case:inbox (mew-case-folder case inbox))
 	 (tag (mew-imap-passtag2 case))
@@ -1484,14 +1546,14 @@
       (save-excursion
 	(set-buffer case:inbox)
 	(when (and (mew-summary-exclusive-p 'no-msg)
-		   (and mew-use-cached-passwd
+		   (and (or mew-use-cached-passwd mew-use-master-passwd)
 			(setq passwd (mew-passwd-get-passwd tag))))
 	  (mew-imap-retrieve case 'biff case:inbox passwd))))))
 
 (defun mew-imap-check ()
   "See if messages are arrived by IMAP."
   (interactive)
-  (let* ((case mew-case-input)
+  (let* ((case mew-case)
 	 (inbox (mew-proto-inbox-folder nil case))
 	 (case:inbox (mew-case-folder case inbox)))
     (when (get-buffer case:inbox)
@@ -1515,7 +1577,7 @@
 	 (t2 (nth 1 ent)) ;; may be nil
 	 (alist (nth 2 ent))
 	 (fld (mew-imap-folder case))
-	 (file (mew-expand-folder fld mew-imap-folder-alist-file))
+	 (file (mew-expand-file fld mew-imap-folder-alist-file))
 	 (t1 (mew-file-get-time file)))
     (if (or no-load (and t1 (not (mew-compare-times t1 t2))))
 	alist
@@ -1538,9 +1600,9 @@
 
 (defun mew-imap-folder-load (case)
   (let* ((fld (mew-imap-folder case))
-	 (file (mew-expand-folder fld mew-imap-folder-alist-file))
+	 (file (mew-expand-file fld mew-imap-folder-alist-file))
 	 (mailboxes (mew-lisp-load file))
-	 (file2 (mew-expand-folder fld mew-imap-friend-folder-list-file))
+	 (file2 (mew-expand-file fld mew-imap-friend-folder-list-file))
 	 (friends (mew-lisp-load file2)))
     (mew-imap-folder-set case mailboxes friends 'no-save)))
 
@@ -1618,11 +1680,66 @@
       (mew-summary-folder-cache-clean qfld)
       (mew-imap-retrieve case 'jobs qfld jobs))))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; UTF-7 IMAP
+;;;
+
+(defun mew-imap-utf-7-decode-string (str)
+  (if (or (null mew-cs-utf-16be) (not (string-match "&" str)))
+      str ;; fast path
+    (let ((regex "&[A-Za-z0-9+,]+-")
+	  padn)
+      (with-temp-buffer
+	(mew-set-buffer-multibyte t)
+	(insert str)
+	(goto-char (point-min))
+	(while (re-search-forward regex nil t)
+	  (narrow-to-region (match-beginning 0) (match-end 0))
+	  (goto-char (point-min))
+	  (delete-char 1)
+	  (while (search-forward "," nil t)
+	    (replace-match "/" nil t))
+	  (goto-char (point-max))
+	  (delete-char -1)
+	  (setq padn (- 4 (% (- (point-max) (point-min)) 4)))
+	  (unless (= padn 4)
+	    (insert (make-string padn ?=)))
+	  (base64-decode-region (point-min) (point-max))
+	  (decode-coding-region (point-min) (point-max) mew-cs-utf-16be)
+	  (goto-char (point-max))
+	  (widen))
+	(buffer-substring-no-properties (point-min) (point-max))))))
+
+(defun mew-imap-utf-7-encode-string (str)
+  (if (or (null mew-cs-utf-16be) (string-match "^[ -%'-~]+$" str))
+      str ;; fast path
+    (let ((regex "[^ -%'-~]+"))
+      (with-temp-buffer
+	(mew-set-buffer-multibyte t)
+	(insert str)
+	(goto-char (point-min))
+	(while (re-search-forward regex nil t)
+	  (narrow-to-region (match-beginning 0) (match-end 0))
+	  (encode-coding-region (point-min) (point-max) mew-cs-utf-16be)
+	  (base64-encode-region (point-min) (point-max))
+	  (goto-char (point-min))
+	  (insert "&")
+	  (when (re-search-forward "=+" nil t)
+	    (delete-region (match-beginning 0) (match-end 0)))
+	  (goto-char (point-min))
+	  (while (search-forward "/" nil t)
+	    (replace-match "," nil t))
+	  (goto-char (point-max))
+	  (insert "-")
+	  (widen))
+	(buffer-substring-no-properties (point-min) (point-max))))))
+
 (provide 'mew-imap)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2002-2003 Mew developing team.
+;; Copyright (C) 2002-2006 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
