@@ -454,23 +454,24 @@
 	 (mew-pop-set-status pnm "quit")
 	 (mew-pop-command-quit pro pnm)
 	 (throw 'write-error nil)))
-      (mew-set-file-modes file)
-      (mew-set-file-type file)
       (mew-pop-set-rcnt pnm (1+ (mew-pop-get-rcnt pnm)))
-      ;;
-      (mew-set-buffer-multibyte t)
-      (setq vec (mew-scan-header))
-      (mew-scan-set-folder vec fld)
-      (mew-scan-set-message vec msg)
-      (if (or (eq directive 'inc) (eq directive 'scan))
-	  (setq mark (mew-scan-inbox-action vec case)))
-      (if (and mark
-	       (eq directive 'scan)
-	       (not (memq mark (list mew-mark-delete mew-mark-unlink))))
-	  (setq mark nil))
-      (mew-scan-body vec)
-      (mew-set-buffer-multibyte nil)
-      (mew-scan-insert-line fld vec width lmsg mark)
+      (when (file-exists-p file)
+	(mew-set-file-modes file)
+	(mew-set-file-type file)
+	;;
+	(mew-set-buffer-multibyte t)
+	(setq vec (mew-scan-header))
+	(mew-scan-set-folder vec fld)
+	(mew-scan-set-message vec msg)
+	(if (or (eq directive 'inc) (eq directive 'scan))
+	    (setq mark (mew-scan-inbox-action vec case)))
+	(if (and mark
+		 (eq directive 'scan)
+		 (not (memq mark (list mew-mark-delete mew-mark-unlink))))
+	    (setq mark nil))
+	(mew-scan-body vec)
+	(mew-set-buffer-multibyte nil)
+	(mew-scan-insert-line fld vec width lmsg mark))
       (mew-pop-set-rtrs pnm (cdr rtrs))
       (if (and del (not truncated))
 	  (progn
