@@ -97,13 +97,18 @@ effect to this function."
   (let ((ct (MEW-CT))
 	(case-fold-search t))
     (cond
-     ((mew-scan-message-invalidp)                  "#")
-     ((mew-scan-message-truncatedp)                "T")
-     ((string-match "Multipart/Signed"    ct)      "S")
-     ((string-match "Multipart/Encrypted" ct)      "E")
-     ((string-match "Application/X-Pkcs7-Mime" ct) "E")
-     ((mew-ct-multipartp ct)                       "M")
-     ((string-match "Message/Partial"     ct)      "P")
+     ((mew-scan-message-invalidp)         "#")
+     ((mew-scan-message-truncatedp)       "T")
+     ((string-match mew-ct-mls  ct)       "S")
+     ((string-match mew-ct-mle  ct)       "E")
+     ((string-match mew-ct-sms  ct)       "E")
+     ((string-match mew-ct-xsms ct)       "E")
+     ((or (string-match mew-ct-smm ct) (string-match mew-ct-xsmm ct))
+      (cond
+       ((string-match mew-ct-smm-sig ct)  "S")
+       ((string-match mew-ct-smm-enc ct)  "E")))
+     ((mew-ct-multipartp ct)              "M")
+     ((string-match "Message/Partial" ct) "P")
      (t " "))))
 
 (defun mew-summary-form-time ()
