@@ -15,6 +15,7 @@
 (defvar mew-prog-spotlight "mdfind")
 (defvar mew-prog-wds       "wdsgrep.exe")
 (defvar mew-prog-google    "gdgrep.rb")
+(defvar mew-prog-est       "estcmd")
 (defvar mew-prog-namazu    "namazu")
 
 (defvar mew-search-switch
@@ -33,6 +34,11 @@
      mew-google-index-folder mew-google-index-all
      mew-pick-canonicalize-pattern-google
      mew-google-register mew-google-unregister)
+    (est "Hyperestraier" ,mew-prog-est
+     mew-search-with-est mew-search-virtual-with-est
+     mew-est-index-folder mew-est-index-all
+     mew-pick-canonicalize-pattern-est
+     nil nil)
     (namazu "Namazu" ,mew-prog-namazu
      mew-search-with-namazu nil
      mew-nmz-index-folder nil
@@ -139,8 +145,9 @@ with a search method."
 	(if ask-folder
 	    (setq flds (mew-input-folders (mew-summary-folder-name))))
 	(setq opattern (mew-input-pick-pattern (concat name " virtual")))
-	(when (and canon-func (fboundp canon-func))
-	  (setq pattern (funcall canon-func opattern)))
+	(if (and canon-func (fboundp canon-func))
+	    (setq pattern (funcall canon-func opattern))
+	  (setq pattern opattern))
 	(setq vfolder (mew-folder-to-virtual opattern))
 	(mew-summary-switch-to-folder vfolder)
 	(mew-sinfo-set-find-key opattern)
@@ -504,6 +511,112 @@ with a search method."
 (defun mew-nmz-index-folder (folder)
   (mew-nmz-mknmz folder nil 'msg))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Hyper Estraier
+;;;
+
+(defvar mew-search-est-db "casket")
+(defvar mew-prog-est-update "mewest")
+
+(defun mew-search-est (pattern path)
+  (setq pattern (mew-cs-encode-string pattern 'utf-8))
+  (let* ((ent (mew-search-get-ent mew-search-method))
+	 (prog (mew-search-get-prog ent))
+	 (casket (expand-file-name mew-search-est-db mew-conf-path))
+	 maildir len attr)
+    (mew-plet
+     (mew-alet
+      (setq maildir (expand-file-name mew-home))
+      (setq len (length (file-name-as-directory maildir)))
+      (setq path (substring path len))
+      (setq path (file-name-as-directory path))
+      (setq attr (format "@uri STRINC %s" (mew-q-encode-string path ?%)))
+      (call-process prog nil t nil "search" "-vu" "-max" "-1" "-ord" "@cdate NUMA" "-attr" attr casket pattern)))))
+
+(defun mew-search-with-est (pattern folder)
+  (let* ((path (mew-expand-folder folder))
+	 (regex (format "file://.*/%s/.*/\\([0-9]+\\)\\(%s\\)?$" 
+			(file-name-nondirectory mew-mail-path)
+			(regexp-quote mew-suffix)))
+	 msgs)
+    (with-temp-buffer
+      (mew-set-buffer-multibyte t)
+      (mew-search-est pattern path)
+      (goto-char (point-min))
+      (while (re-search-forward regex nil t)
+	(setq msgs (cons (mew-match-string 1) msgs))
+	(forward-line))
+      (setq msgs (nreverse msgs))
+      (setq msgs (sort (mapcar 'string-to-number msgs) '<))
+      (mapcar 'number-to-string msgs))))
+
+(defun mew-search-virtual-with-est (pattern flds)
+  (let* ((regex (format "file://.*/%s/\\(.*\\)/\\([0-9]+\\)\\(%s\\)?$" 
+			(file-name-nondirectory mew-mail-path)
+			(regexp-quote mew-suffix)))
+	 (file (mew-make-temp-name))
+	 (prev "") (rttl 0) (n 1)
+	 path crnt start)
+    (unless flds (setq flds (list mew-folder-local)))
+    (setq flds (mew-expand-wildcard-folder flds))
+    (while flds
+      (setq path (mew-expand-folder (car flds)))
+      (setq flds (cdr flds))
+      (mew-search-est pattern path)
+      (goto-char (point-min))
+      (setq start (point))
+      (while (re-search-forward regex nil t)
+	(setq rttl (1+ rttl))
+	(setq crnt (match-string 1))
+	(delete-region start (match-beginning 2))
+	(when (not (string= crnt prev))
+	  (beginning-of-line)
+	  (insert "CD:" mew-folder-local (mew-q-decode-string crnt ?%) "\n"))
+	(setq prev crnt)
+	(forward-line)
+	(setq start (point)))
+      (delete-region start (point-max))
+      (mew-frwlet
+       mew-cs-text-for-read mew-cs-text-for-write
+       (write-region (point-min) (point-max) file (> n 1) 'no-msg))
+      (mew-erase-buffer)
+      (setq n (1+ n)))
+    (list file rttl)))
+
+(defun mew-est-index-folder (folder)
+  "Make Hyper Estraier index for this folder."
+  (interactive)
+  (if (not (mew-which-exec mew-prog-est-update))
+      (message "\"%s\" does not exist" mew-prog-est-update)
+    (message "Hyper Estraier indexing for %s..." folder)
+    (let* ((path (file-truename (mew-expand-folder folder)))
+	   (pro (start-process "*Mew EST*" nil mew-prog-est-update path)))
+      (set-process-filter pro 'mew-est-index-filter)
+      (set-process-sentinel pro 'mew-est-index-sentinel))))
+
+(defun mew-est-index-all ()
+  "Make Hyper Estraier index for all folders."
+  (interactive)
+  (if (not (mew-which-exec mew-prog-est-update))
+      (message "")
+    (message "Hyper Estraier indexing...")
+    (let ((pro (start-process "*Mew EST*" nil mew-prog-est-update)))
+      (set-process-filter pro 'mew-est-index-filter)
+      (set-process-sentinel pro 'mew-est-index-sentinel))))
+
+(defun mew-est-index-filter (process string)
+  (cond
+   ((string-match "exists" string)
+    (message "Another Hyper Estraier indexer is running"))
+   ((string-match "\.\.\.failed" string)
+    (message "Hyper Estraier indexing ...failed"))
+   ((string-match "\.\.\.done" string)
+    (message "Hyper Estraier indexing ...done"))))
+
+(defun mew-est-index-sentinel (process event)
+  )
+
 (provide 'mew-search)
 
 ;;; Copyright Notice:
