@@ -347,23 +347,25 @@ sorted. But if you do so, '\\<mew-summary-mode-map>\\[mew-summary-ls]' + 'sync'
      (mew-syntax-change-message-number2 dst)
      (forward-line))))
 
-(defun mew-summary-pack ()
+(defun mew-summary-pack (&optional force)
   "Pack messages and list them up again.
 After packing, the cursor stays in the current message.
 If this command is used in a remote folder,
 local cache messages are packed."
-  (interactive)
-  (mew-summary-only
-   (mew-summary-local-only
-    (mew-summary-not-in-queue
-     (mew-summary-not-in-draft
-      (when (mew-summary-exclusive-p)
-	(let ((folder (mew-summary-folder-name)))
-	  (cond
-	   ((null folder)
-	    (message "No message"))
-	   ((or (not mew-ask-pack) (y-or-n-p (format "Pack %s? " folder)))
-	    (mew-summary-pack-body folder))))))))))
+  (interactive "P")
+  (if (not force)
+      (message "This command was obsoleted. Why are you using pack?")
+    (mew-summary-only
+     (mew-summary-local-only
+      (mew-summary-not-in-queue
+       (mew-summary-not-in-draft
+	(when (mew-summary-exclusive-p)
+	  (let ((folder (mew-summary-folder-name)))
+	    (cond
+	     ((null folder)
+	      (message "No message"))
+	     ((or (not mew-ask-pack) (y-or-n-p (format "Pack %s? " folder)))
+	      (mew-summary-pack-body folder)))))))))))
 
 (defun mew-summary-pack-body (folder)
   (let* ((dir (mew-expand-folder folder))
