@@ -19,7 +19,7 @@
     "scan-id" "scan-md5" "find-key" "cursor-line" "direction" "start-point"
     "cache-time" "summary-form" "unread-mark" "refile-back"
     "disp-msg" "case" "folder" "proto" "mid-marker" "mid-line" "ttl-line"
-    "mark-review" "ret-pos" "inboxp"))
+    "mark-review" "ret-pos" "inboxp" "column"))
 
 (mew-blinfo-defun 'mew-sinfo mew-sinfo-list)
 
