@@ -221,10 +221,6 @@ See 'mew-imap-spam-field'."
   :type '(choice (const :tag "Not use" nil)
 		 (string :tag "Spam folder")))
 
-;; must be here
-(defvar mew-basic-folders
-  (list mew-inbox-folder mew-draft-folder mew-queue-folder mew-postq-folder))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Hooks
@@ -437,7 +433,7 @@ on a draft buffer. If nil, this value is automatically generated from
 
 (defcustom mew-mail-domain-list nil
   "*Your e-mail address domain list like 
-(\"Mew.org\" \"iijlab.net\").
+(\"example.org\" \"example.jp\").
 They are used for mail-domain completion in Draft mode(C-cTAB). If
 nil, this value is automatically generated from 'mew-config-alist'."
   :group 'mew-draft
@@ -450,29 +446,25 @@ also 'mew-from'."
   :group 'mew-draft
   :type '(choice (const nil) (repeat string)))
 
-(defcustom mew-my-domains nil
-  "*A list of my domains.
-If addresses on 'mew-draft-address-warning-fields' in a draft
-are not matched to this domain lists, they
-turns the warning color.
-Domains should be downcased."
-  :group 'mew-draft
-  :type '(choice (const nil) (repeat string)))
-
 (defcustom mew-warn-addresses nil
-  "*A list of warning addresses even if it has my domains.
-If addresses on 'mew-draft-address-warning-fields' in a draft
-are matched to this address lists, they turns the warning color.
-Addresses should be downcased."
+  "A list of addresses to be warned."
   :group 'mew-draft
   :type '(choice (const nil) (repeat string)))
 
-(defcustom mew-my-address-no-warning nil
-  "*If non-nil, my address in a draft does not
-turn the warning color even if its domain
-is not included 'mew-my-domains'."
+(defcustom mew-safe-addresses nil
+  "A list of addresses not to be warned."
   :group 'mew-draft
-  :type 'boolean)
+  :type '(choice (const nil) (repeat string)))
+
+(defcustom mew-warn-domains nil
+  "A list of domains to be warned."
+  :group 'mew-draft
+  :type '(choice (const nil) (repeat string)))
+
+(defcustom mew-safe-domains nil
+  "A list of domains not to be warned."
+  :group 'mew-draft
+  :type '(choice (const nil) (repeat string)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; 
@@ -579,13 +571,14 @@ use a string. (e.g. \"25\")"
   :type 'boolean)
 
 (defcustom mew-smtp-ssl-port "465" ;; no universal consensus
-  "*The port for SMTP over SSL/TLS."
+  "*The port for SMTP over SSL/TLS. Set this to \"smtp\" if you 
+want to use TLS."
   :group 'mew-smtp
   :type 'string)
 
 (defcustom mew-smtp-user nil
-  "*The user name on your SMTP server. If non-nil, authentication is
-used."
+  "*The user name on your SMTP server. If not configured, 
+your e-mail address is automatically set"
   :group 'mew-smtp
   :type 'string)
 
@@ -1219,9 +1212,9 @@ element must be 'name, 'comment, 'address, 'nickname, or appropriate
 
 Consider the following examples:
 
-	A: Kazu Yamamoto <kazu@mew.org>
-	B: kazu@mew.org (Kazu Yamamoto)
-	C: Kazuhiko Yamamoto (Kazu) <kazu@mew.org>
+	A: Kazu Yamamoto <kazu@example.org>
+	B: kazu@example.org (Kazu Yamamoto)
+	C: Kazuhiko Yamamoto (Kazu) <kazu@example.org>
 
 Each element returns the following value:
 
@@ -1234,9 +1227,9 @@ Each element returns the following value:
 				B: Kazu Yamamoto
 				C: (Kazu)
 	'address	Address part
-				A: kazu@mew.org
-				B: kazu@mew.org
-				C: kazu@mew.org
+				A: kazu@example.org
+				B: kazu@example.org
+				C: kazu@example.org
 
 	'nickname       One element of personal information in
                         Addrbook according to 'mew-addrbook-for-summary'.
@@ -1263,6 +1256,16 @@ instead of the 'D' mark."
 		 (const :tag "Not use" nil)
 		 (character :tag "Use another character")))
 
+(defcustom mew-summary-form-mark-spam nil
+  "*If non-nil, the 'D' mark automatically is put onto
+SPAM messages. A message is considered a SPAM message 
+if the MD5 checksum of its body is duplicated in a scan.
+If a character, use it as a mark instead of the 'D' mark."
+  :group 'mew-summary
+  :type '(choice (const :tag "Use mew-mark-delete" t)
+		 (const :tag "Not use" nil)
+		 (character :tag "Use another character")))
+
 (defcustom mew-summary-form-mark-review nil
   "*If non-nil, the '*' mark automatically is put onto
 messages destined to me. If a character, use it as a mark
@@ -1287,29 +1290,12 @@ quantization by 'mew-summary-form-size-unit'."
 
 (defcustom mew-spam: "X-Bogosity:"
   "*The field name which your spam filter inserts. 
-This variable MUST be set in \".emacs\".
 E.g.
 	\"X-Spam-Flag:\" for 'spamassassin'
 	\"X-Bogosity:\" for 'bogofilter'"
   :group 'mew-summary
   :type 'string)
 
-(defcustom mew-scan-fields
-  (list "Folder:" "Filename:" 
-	mew-subj: mew-date: mew-from: mew-to: mew-cc:
-	mew-ct: mew-cte: mew-x-mew-uidl:
-	mew-message-id: mew-in-reply-to: mew-references: mew-x-mew-ref:
-	mew-spam: "Body")
-  "*A list which specifies mewl's output.
-The first element MUST be \"Folder:\".
-The second element MUST be \"Filename:\".
-Both \"In-Reply-To:\" and \"References:\" MUST be included for thread.
-The last MUST be \"Body\"."
-  :group 'mew-summary
-  :type '(cons (const "Folder:")
-               (cons (const "Filename:")
-                     (repeat string))))
-
 (defvar mew-scan-fields-alias
   '("FLD" "NUM" "SUBJ" "DATE" "FROM" "TO" "CC"
     "CT" "CTE" "UID"
@@ -1423,6 +1409,13 @@ to this file."
   :group 'mew-env
   :type 'integer)
 
+(defcustom mew-scan-body-length 10
+  "*Body length which Mew treats.
+Increase this value if 'mew-summary-form-mark-spam' put
+the 'D' mark onto different messages."
+  :group 'mew-env
+  :type 'integer)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Ask?
@@ -1491,11 +1484,6 @@ invalid. Otherwise, 'update is used for range in the case."
   :group 'mew-summary
   :type 'boolean)
 
-(defcustom mew-ask-virtual-folder-name nil
-  "*If *non-nil*, ask a virtual folder name."
-  :group 'mew-summary
-  :type 'boolean)
-
 (defcustom mew-ask-charset nil
   "*Mew asks a user whether or not the charset chosen on composing
 is appropriate.
@@ -1616,6 +1604,11 @@ created (if the feature is provided)."
   :group 'mew-privacy
   :type 'boolean)
 
+(defcustom mew-use-master-passwd nil
+  "*if non-nil, a master password is used."
+  :group 'mew-privacy
+  :type 'boolean)
+
 (defcustom mew-use-8bit nil
   "*If non-nil, the 8bit-clean charset mechanism is used for 8bit
 charsets, such as ISO-8859-1, in Draft mode. If nil, an appropriate
@@ -1759,6 +1752,21 @@ the unread mark is deleted."
   :group 'mew-summary
   :type 'boolean)
 
+(defcustom mew-use-smtp-auth t
+  "*If nil, SMTP AUTH is ignored even an SMTP server
+returns the AUTH capability."
+  :group 'mew-smtp
+  :type 'boolean)
+
+(defcustom mew-smtp-auth-plain-authorize-id nil
+  "*If nil, SMTP AUTH PLAIN is created as follow:
+	NUL authenticate-id NUL password
+If non-nil, it is created as follow:
+	authorize-id NUL authenticate-id NUL password
+For more information, see RFC 2595."
+  :group 'mew-smtp
+  :type 'boolean)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Appearance
@@ -1824,6 +1832,12 @@ See also 'mew-summary-show-direction'."
   :group 'mew-summary
   :type '(choice (const next) (const up) (const down) (const stop)))
 
+(defcustom mew-summary-show-pause nil
+  "*Pause on going to next message if *non-nil*.
+See also 'mew-summary-show-direction' and 'mew-summary-mark-direction'."
+  :group 'mew-summary
+  :type 'boolean)
+
 (defcustom mew-summary-mark-duplicated-skip 'first
   "*This value decides which duplicated message should not be
 marked with 'mew-mark-duplicated' when 'mew-summary-mark-duplicated'
@@ -1946,7 +1960,7 @@ not work resulting that no garbage file remains."
 (defcustom mew-header-alist nil
   "*Alist of key and value pair for header field to be inserted on draft.
 '((\"X-fingerprint:\" . \"6B 63 38 88 67 5E 96 8E  CE A4 62 73 3F 11 64 94\")
-  (\"X-URL:\" . \"http://www.Mew.org/~kazu/\"))"
+  (\"X-URL:\" . \"http://www.example.org/\"))"
   :group 'mew-draft
   :type 'sexp)
 
@@ -2308,7 +2322,8 @@ circular domain completion."
 	    mew-date: mew-message-id: mew-mv: "Content-"
 	    "Precedence:" mew-organization: "X-"
 	    "Lines:" "Status:" "Posted:" "Forwarded:" "Replied:" "X-UIDL:"
-	    "DomainKey-Signature:" "Authentication-Results:"))
+	    "DomainKey-Signature:" "DKIM-Signature:"
+	    "Authentication-Results:"))
   "*A field list to be deleted when edit again."
   :group 'mew-draft
   :type '(choice (const nil)
@@ -2318,7 +2333,7 @@ circular domain completion."
   (list mew-received: "Return-Path:" "Delivery-Date:" "Delivered-To:"
 	mew-to: mew-cc: mew-from: mew-organization:
 	"Lines:" "Status:" "Posted:" "Forwarded:" "Replied:" "X-UIDL:"
-	"DomainKey-Signature:")
+	"DomainKey-Signature:" "DKIM-Signature:")
   "*A field list to be deleted when send with different To: and Cc:."
   :group 'mew-draft
   :type '(choice (const nil)
@@ -2838,7 +2853,10 @@ appropriate for this value."
                  (const pgp-encryption)
                  (const pgp-signature-encryption)
 		 (const pgp-encryption-signature)
-		 (const smime-signature) ;; xxx
+		 (const smime-signature)
+                 (const smime-encryption)
+                 (const smime-signature-encryption)
+		 (const smime-encryption-signature)
                  (other :tag "nil" nil)))
 
 (defcustom mew-protect-privacy-encrypted nil
@@ -3033,7 +3051,7 @@ w3 is used."
 ;;;
 
 
-(defvar mew-file-type "MHTX")
+(defvar mew-file-type "MewX")
 (defvar mew-file-creator "EMAx")
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -3048,10 +3066,10 @@ w3 is used."
 
 Good example:
 	From: Kazu Yamamoto (=?ISO-2022-JP?B?GyRCOzNLXE9CSScbKEI=?=) 
-		<kazu@iijlab.net>
+		<kazu@example.org>
 Bad example:
 	From: Kazu Yamamoto (\"=?ISO-2022-JP?B?GyRCOzNLXE9CSScbKEI=?=\") 
-		<kazu@iijlab.net>
+		<kazu@example.org>
 
 (2) Decode raw non-ASCII text, which violates RFC 822.
 This is typically found in Subject: field.
@@ -3133,11 +3151,24 @@ Level 2: syntax error."
   :group 'mew-draft
   :type 'hook)
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Preventing waring messages when byte-compiling
+;;;
+
+(defvar mew-regex-message-files  nil)
+(defvar mew-regex-message-files2 nil)
+(defvar mew-regex-message-files3 nil)
+(defvar mew-regex-message-files4 nil)
+(defvar mew-regex-message-files5 nil)
+
+(defvar mew-basic-folders nil)
+
 (provide 'mew-vars)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2005 Mew developing team.
+;; Copyright (C) 1996-2006 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
