@@ -514,7 +514,12 @@ the next version of Mew.")
      ((,mew-ct-mls ,mew-ct-pgs) (,mew-ct-mle ,mew-ct-pge)) "PSPE")
     (pgp-encryption-signature
      ((,mew-ct-mle ,mew-ct-pge) (,mew-ct-mls ,mew-ct-pgs)) "PEPS")
-    (smime-signature  ((,mew-ct-mls ,mew-ct-sms)) "SS"))
+    (smime-signature  ((,mew-ct-smm ,mew-ct-smm-sig)) "SS")
+    (smime-encryption ((,mew-ct-smm ,mew-ct-smm-enc)) "SE")
+    (smime-signature-encryption
+     ((,mew-ct-smm ,mew-ct-smm-sig) (,mew-ct-smm ,mew-ct-smm-enc)) "SSSE")
+    (smie-encryption-signature
+     ((,mew-ct-smm ,mew-ct-smm-enc) (,mew-ct-smm ,mew-ct-smm-sig)) "SESS"))
   "*Alist of key, a list of privacy Content-Type, and its mark."
   :group 'mew-privacy
   :type 'sexp)
